from ..models import BaseModel
from typing import List


class BaseExtractor:
    def __init__(
        self,
        name: str,  # Has to be unique within graph as it identifies extractor
        query: str,
        model: BaseModel,
        max_chunk_size: float = 10_000,
        first_chunk_only: bool = False,
    ) -> None:
        self.max_chunk_size = max_chunk_size
        self.model = model
        self.query = query
        self.name = name
        self.first_chunk_only = first_chunk_only

    def _chunk_text(self, doc_text: str) -> List[str]:
        chunks = doc_text.split("\n")

        merged_chunks = [""]
        for chunk in chunks:
            prev_chunk = merged_chunks[-1]
            if (len(prev_chunk) + len(chunk)) < (self.max_chunk_size - 1):
                merged_chunks[-1] = prev_chunk + "\n" + chunk
            else:
                merged_chunks.append(chunk)

        # In some cases it can be useful to only provide the opening of the document
        # E.g. when classifying the document
        if self.first_chunk_only:
            return [merged_chunks[0]]

        return merged_chunks

    def extract(self, doc_text: str):
        raise NotImplementedError



from ..utils import parseNumber, most_common
from ..models import BaseModel
from .base import BaseExtractor
from typing import List, Optional
import re
import numpy as np

TASK_DESCRIPTION = "Use the information given below."

SYSTEM_PROMPT = 'Answer only with the relevant text snippet you have found below, and no other text. Do not explain your answer or provide any context. If there is no relevant information in the text provided, respond with "NA". Do not make things up.'


class TextExtractor(BaseExtractor):

    def extract(self, doc_text: str) -> str:
        merged_chunks = self._chunk_text(doc_text)

        if self.model.model_description()["scores"]:
            results_with_scores = self.model.batch_complete_with_scores(
                query=self.query,
                context=merged_chunks,
                task_description=TASK_DESCRIPTION,
                system_prompt=SYSTEM_PROMPT,
            )
            all_scores = [r["score"] for r in results_with_scores]
            idx = np.argmax(all_scores)
            consensus = results_with_scores[idx]["answer"]
            return consensus
        else:
            results = self.model.batch_complete(
                query=self.query,
                context=merged_chunks,
                task_description=TASK_DESCRIPTION,
                system_prompt=SYSTEM_PROMPT,
            )
            valid_answers = []
            valid_indices = []
            for idx, answer in enumerate(results):
                if answer != "NA":
                    valid_answers.append(answer)
                    valid_indices.append(idx)

            if len(valid_answers) > 0:
                consensus = most_common(valid_answers)
            else:
                consensus = "NA"
            return consensus
